import 'dart:async';
import 'package:bloc/bloc.dart';
import 'package:calendar_view/calendar_view.dart';
import 'package:flutter/material.dart';
part 'events_event.dart';
part 'events_state.dart';

class CalendarEventsBloc extends Bloc<CalendarEventsEvent, CalendarEventState> {
  final EventController eventController = EventController();

  CalendarEventsBloc() : super(EventsInitial()) {
    on<LoadEvents>(_onLoadEvents);
    on<AddEvent>(_onAddEvent);
    on<StartTimer>(_onStartTimer);
    on<DisposeResources>(_onDisposeResources);
    on<GoToWeek>(_onGoToWeek);
  }

  Future<void> _onLoadEvents(
    LoadEvents event,
    Emitter<CalendarEventState> emit,
  ) async {
    emit(EventsLoading());
    try {
      final events = _generateDummyEventsForWeek(event.weekStart);
      eventController.addAll(events);
      emit(EventsLoaded(
        events: events,
        initialDate: event.weekStart,
        weekDays: _getWeekDays(event.weekStart),
      ));
    } catch (e) {
      emit(EventsError('Failed to load events'));
    }
  }

  List<CalendarEventData> _generateDummyEventsForWeek(DateTime weekStart) {
    final events = <CalendarEventData>[];

    for (int i = 0; i < 7; i++) {
      final date = weekStart.add(Duration(days: i));

      events.add(CalendarEventData(
        date: date,
        startTime: date.copyWith(hour: 9, minute: 0),
        endTime: date.copyWith(hour: 10, minute: 30),
        title: 'Team Meeting',
        description: 'Daily standup',
        color: Colors.blue,
      ));
      events.add(CalendarEventData(
        date: date,
        startTime: date.copyWith(hour: 14, minute: 0),
        endTime: date.copyWith(hour: 15, minute: 0),
        title: 'Client Call',
        description: 'Project discussion',
        color: Colors.green,
      ));
    }

    return events;
  }

  void _onAddEvent(AddEvent event, Emitter<CalendarEventState> emit) {
    eventController.add(event.event);
    if (state is EventsLoaded) {
      final loaded = state as EventsLoaded;
      emit(EventsLoaded(
        events: [...eventController.events],
        initialDate: loaded.initialDate,
        weekDays: loaded.weekDays,
      ));
    }
  }

  void _onStartTimer(StartTimer event, Emitter<CalendarEventState> emit) {}

  void _onDisposeResources(
      DisposeResources event, Emitter<CalendarEventState> emit) {
    eventController.dispose();
  }

  void _onGoToWeek(GoToWeek event, Emitter<CalendarEventState> emit) {
    if (state is EventsLoaded) {
      final loaded = state as EventsLoaded;
      final newWeekDays = _getWeekDays(event.weekDate);
      emit(EventsLoaded(
        events: loaded.events,
        initialDate: event.weekDate,
        weekDays: newWeekDays,
      ));
    }
  }

  List<CalendarEventData> _generateDummyEvents() {
    final now = DateTime.now();
    return [
      CalendarEventData(
        date: now,
        startTime: now.copyWith(hour: 8, minute: 00, second: 0),
        endTime: now.copyWith(hour: 9, minute: 00, second: 0),
        title: 'Team Meeting',
        description: 'Weekly team sync',
        color: Colors.blue,
      ),
      CalendarEventData(
        date: now,
        startTime: now.copyWith(hour: 9, minute: 00, second: 0),
        endTime: now.copyWith(hour: 10, minute: 30, second: 0),
        title: 'Team Meeting',
        description: 'Weekly team sync',
        color: Colors.blue,
      ),
      CalendarEventData(
        date: now.add(const Duration(days: 1)),
        startTime: now.copyWith(hour: 14, day: now.day + 1),
        endTime: now.copyWith(hour: 15, day: now.day + 1),
        title: 'Client Call',
        description: 'Project discussion',
        color: Colors.green,
      ),
      CalendarEventData(
        date: now.add(const Duration(days: 2)),
        startTime: now.copyWith(hour: 11, day: now.day + 2),
        endTime: now.copyWith(hour: 12, day: now.day + 2),
        title: 'Lunch with Team',
        color: Colors.orange,
      ),
    ];
  }

  List<DateTime> _getWeekDays(DateTime date) {
    final int weekday = date.weekday;
    final DateTime monday = date.subtract(Duration(days: weekday - 1));
    return List.generate(7, (i) => monday.add(Duration(days: i)));
  }

  @override
  Future<void> close() {
    eventController.dispose();
    return super.close();
  }
}
