import 'package:flutter/material.dart';
import 'package:calendar_date_picker2/calendar_date_picker2.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class CustomCalendarPage extends StatefulWidget {
  final DateTime selectedDate;
  final Function(int day, int month, int year) onDateChanged;

  const CustomCalendarPage({
    super.key,
    required this.selectedDate,
    required this.onDateChanged,
  });

  @override
  State<CustomCalendarPage> createState() => _CustomCalendarPageState();
}

class _CustomCalendarPageState extends State<CustomCalendarPage> {
  late DateTime _selectedDate;

  @override
  void initState() {
    super.initState();
    _selectedDate = widget.selectedDate;
  }

  @override
  void didUpdateWidget(CustomCalendarPage oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.selectedDate != oldWidget.selectedDate) {
      setState(() {
        _selectedDate = widget.selectedDate;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    final config = CalendarDatePicker2Config(
      calendarType: CalendarDatePicker2Type.single,
      selectedDayHighlightColor: const Color(0xFF3B82F6),
      selectedDayTextStyle: const TextStyle(
        color: Colors.white,
        fontWeight: FontWeight.w400,
        fontSize: 14,
      ),
      dayTextStyle: const TextStyle(
        color: ColorsManager.blackColor,
        fontWeight: FontWeight.w400,
        fontSize: 14,
      ),
      weekdayLabelTextStyle: const TextStyle(
        color: ColorsManager.grey50,
        fontWeight: FontWeight.w400,
        fontSize: 14,
      ),
      controlsTextStyle: const TextStyle(
        color: Color(0xFF232D3A),
        fontWeight: FontWeight.w400,
        fontSize: 18,
      ),
      centerAlignModePicker: false,
      disableMonthPicker: true,
      firstDayOfWeek: 1,
      weekdayLabels: const ['Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su'],
    );

    return CalendarDatePicker2(
      config: config,
      value: [_selectedDate],
      onValueChanged: (dates) {
        final picked = dates.first;
        if (picked != null) {
          setState(() {
            _selectedDate = picked;
          });
          widget.onDateChanged(picked.day, picked.month, picked.year);
        }
      },
    );
  }
}
